import 'package:fis_common/event/event_type.dart';

import 'adapters/core.dart';
import 'channel.dart';
import 'exception.dart';
import 'source.dart';

/// 直播控制器
class LiveController {
  /// 直播源
  final LiveSource source;

  /// 频道配置
  final LiveChannelConfig? channelConfig;

  /// 控制适配器
  ILiveControllerAdapter? adapter;

  /// 播放
  void play() => adapter?.play();

  /// 停止
  void stop() => adapter?.stop();

  /// 就绪事件
  final ready = FEventHandler<void>();

  /// 超时事件
  final timeout = FEventHandler<void>();

  /// 播放开始事件
  final playStarted = FEventHandler<void>();

  /// 播放停止事件
  final playStopped = FEventHandler<void>();

  /// 正在缓冲事件
  final buffering = FEventHandler<void>();

  /// 发生错误事件
  final errorOccurred = FEventHandler<FLivePlayerException>();

  LiveController({
    required this.source,
    this.channelConfig,
    this.adapter,
  });
}
