import 'package:flutter/material.dart';

import '../controller/controller.dart';
import 'data_root.dart';
import 'crop/channel_view.dart';
import 'single/single_view.dart';
import 'style.dart';

/// 直播播放器
class FLivePlayer extends StatefulWidget {
  /// 控制器
  final LiveController controller;

  /// 直播样式
  final LivePlayerStyle? style;

  const FLivePlayer({
    super.key,
    required this.controller,
    this.style,
  });

  @override
  State<StatefulWidget> createState() => _FLivePlayerState();
}

class _FLivePlayerState extends State<FLivePlayer> {
  static const double kLoadingSize = 32;
  late final LiveController controller;
  bool _isReady = false;

  @override
  void initState() {
    controller = widget.controller;
    controller.ready.addListener(_onReady);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.black, // 由于播放器第一帧未显示时会有黑屏，默认背景色为黑色
      child: Stack(
        children: [
          _buildLiveView(),
          if (!_isReady)
            const Center(
              child: SizedBox(
                width: kLoadingSize,
                height: kLoadingSize,
                child: CircularProgressIndicator(),
              ),
            ),
        ],
      ),
    );
  }

  Widget _buildLiveView() {
    final isSingle = widget.controller.channelConfig == null;
    return LiveShareDateProxy(
      controller: widget.controller,
      style: widget.style,
      child: isSingle ? const SingleChannelView() : const CropChannelView(),
    );
  }

  void _onReady(Object sender, void e) {
    setState(() {
      _isReady = true;
    });
  }
}
