import 'dart:convert';

import 'package:fis_live/controller/adapters/single/adapter_web.dart';
import 'package:fis_live/controller/controller.dart';
import 'package:fis_live/controller/exception.dart';
import 'package:fis_live/utils/asset_util.dart';
import 'package:fis_live/widgets/style.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:webviewx/webviewx.dart';

import '../data_root.dart';

/// 单路画面直播视图
class SingleChannelView extends StatefulWidget {
  const SingleChannelView({super.key});

  @override
  State<SingleChannelView> createState() => _SingleChannelViewState();
}

class _SingleChannelViewState extends State<SingleChannelView> {
  LiveShareDateProxy? dataProxy;

  LiveController get controller => dataProxy!.controller;
  LiveControllerSingleWebAdapter get adapter =>
      controller.adapter! as LiveControllerSingleWebAdapter;
  WebViewXController get wvController => adapter.webViewController!;
  LivePlayerStyle? get style => dataProxy!.style;

  @override
  Widget build(BuildContext context) {
    _loadData();

    return LayoutBuilder(
      builder: (_, constraints) {
        return FutureBuilder<String>(
          future: rootBundle.loadString(
            AssetUtil.getPath("wwwroot/source.html"),
          ),
          builder: (_, snapshot) {
            if (snapshot.connectionState == ConnectionState.done &&
                snapshot.data != null &&
                snapshot.data!.isNotEmpty) {
              // !!! 【Important】Web端js资源路径和Native不一样，需要特殊处理一下
              final webJsAssetPath = AssetUtil.getPath(
                  "wwwroot/node_player_beta/NodePlayer.min.js");
              String content = snapshot.data!;
              content = content.replaceFirst("./node_player/NodePlayer.min.js",
                  "./assets/$webJsAssetPath");
              //<script src="./assets/packages/fis_live/assets/wwwroot/node_player_beta/NodePlayer.min.js"></script>
              return _buildWebView(
                content,
                constraints.maxWidth,
                constraints.maxHeight,
              );
            }
            return const SizedBox();
          },
        );
      },
    );
  }

  Widget _buildWebView(String content, double width, double height) {
    return WebViewX(
      initialSourceType: SourceType.html,
      initialContent: content,
      width: width,
      height: height,
      ignoreAllGestures: true,
      dartCallBacks: {
        DartCallback(name: "onReady", callBack: _onReady),
        DartCallback(name: "onStart", callBack: _onStart),
        DartCallback(name: "onStop", callBack: _onStop),
        DartCallback(name: "onTimeout", callBack: _onTimeout),
        DartCallback(name: "onError", callBack: _onError),
      },
      onWebViewCreated: (c) {
        adapter.webViewController = c;
      },
      onPageFinished: (src) {},
    );
  }

  void _loadData() {
    if (dataProxy == null) {
      dataProxy = LiveShareDateProxy.of(context)!;
      // !!!【Important】 创建适配器
      controller.adapter = LiveControllerSingleWebAdapter();
    }
  }

  dynamic _onReady(dynamic args) {
    final url = controller.source.url;
    wvController.callJsMethod("load", [url]);
    controller.ready.emit(this, null);
  }

  dynamic _onStart(dynamic args) {
    controller.playStarted.emit(this, null);
  }

  dynamic _onStop(dynamic args) {
    controller.playStopped.emit(this, null);
  }

  dynamic _onTimeout(dynamic args) {
    controller.timeout.emit(this, null);
  }

  dynamic _onError(dynamic args) {
    final error = jsonDecode(args as String);
    controller.errorOccurred.emit(
      this,
      FLivePlayerException(error['code'], error['msg']),
    );
  }
}
